@@ -1,9 +1,9 @@
 /* GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -101,7 +100,7 @@ static int
 get_hue (guchar *rgb)
 {
   double h, v, temp, diff;
-  /* TODO : There seems to be some typoes in the comments here.
+  /* TODO : There seems to be some typos in the comments here.
    * Ask vidar what he meant.
    * */
   if ((rgb[0] == rgb[1]) && (rgb[0] == rgb[2])) /* Gray */
@@ -185,9 +184,9 @@ choose_best_brush (ppm_t *p, ppm_t *a, int tx, int ty,
               if ((h = brush->col[(y * brush->width * 3) + x * 3]))
                 {
                   v = h / 255.0;
-                  dev += abs (row[k+0] - r) * v;
-                  dev += abs (row[k+1] - g) * v;
-                  dev += abs (row[k+2] - b) * v;
+                  dev += fabs (row[k+0] - r) * v;
+                  dev += fabs (row[k+1] - g) * v;
+                  dev += fabs (row[k+2] - b) * v;
                   if (img_has_alpha)
                     dev += a->col[(ty + y) * a->width * 3 + (tx + x) * 3] * v;
                 }
@@ -206,7 +205,7 @@ choose_best_brush (ppm_t *p, ppm_t *a, int tx, int ty,
         {
           best = i;
           bestdev = dev;
-          brlist = g_list_append (brlist, (void *)i);
+          brlist = g_list_append (brlist, GINT_TO_POINTER (i));
         }
       if (dev < runningvals.devthresh)
         break;
@@ -219,7 +218,7 @@ choose_best_brush (ppm_t *p, ppm_t *a, int tx, int ty,
     }
 
   i = g_rand_int_range (random_generator, 0, g_list_length (brlist));
-  best = (long)((g_list_nth (brlist,i))->data);
+  best = GPOINTER_TO_INT ((g_list_nth (brlist,i))->data);
   g_list_free (brlist);
 
   return best;
@@ -354,7 +353,7 @@ repaint (ppm_t *p, ppm_t *a)
   int         tx = 0, ty = 0;
   ppm_t       tmp = {0, 0, NULL};
   ppm_t       atmp = {0, 0, NULL};
-  int         r, g, b, n, h, i, j, on, sn;
+  int         r, g, b, h, i, j, on, sn;
   int         num_brushes, maxbrushwidth, maxbrushheight;
   guchar      back[3] = {0, 0, 0};
   ppm_t      *brushes, *shadows;
@@ -362,13 +361,11 @@ repaint (ppm_t *p, ppm_t *a)
   double     *brushes_sum;
   int         cx, cy, maxdist;
   double      scale, relief, startangle, anglespan, density, bgamma;
-  double      thissum;
   int         max_progress;
   ppm_t       paper_ppm = {0, 0, NULL};
   ppm_t       dirmap = {0, 0, NULL};
   ppm_t       sizmap = {0, 0, NULL};
   int        *xpos = NULL, *ypos = NULL;
-  int         step = 1;
   int         progstep;
   static int  running = 0;
 
@@ -467,7 +464,6 @@ repaint (ppm_t *p, ppm_t *a)
     }
 
   brush = &brushes[0];
-  thissum = brushes_sum[0];
 
   maxbrushwidth = maxbrushheight = 0;
   for (i = 0; i < num_brushes; i++)
@@ -547,8 +543,7 @@ repaint (ppm_t *p, ppm_t *a)
       guchar tmpcol[3];
 
       ppm_new (&tmp, p->width, p->height);
-      gimp_rgb_get_uchar (&runningvals.color,
-                          &tmpcol[0], &tmpcol[1], &tmpcol[2]);
+      gegl_color_get_pixel (runningvals.color, babl_format ("R'G'B' u8"), tmpcol);
       fill (&tmp, tmpcol);
     }
   else if (runningvals.general_background_type == BG_TYPE_KEEP_ORIGINAL)
@@ -557,22 +552,37 @@ repaint (ppm_t *p, ppm_t *a)
     }
   else
     {
-      scale = runningvals.paper_scale / 100.0;
+      int dx, dy;
+
       ppm_new (&tmp, p->width, p->height);
       ppm_load (runningvals.selected_paper, &paper_ppm);
-      resize (&paper_ppm, paper_ppm.width * scale, paper_ppm.height * scale);
+
+      if (runningvals.paper_scale != 100.0)
+        {
+          scale = runningvals.paper_scale / 100.0;
+          resize (&paper_ppm, paper_ppm.width * scale, paper_ppm.height * scale);
+        }
+
       if (runningvals.paper_invert)
         ppm_apply_gamma (&paper_ppm, -1.0, 1, 1, 1);
-      for (x = 0; x < tmp.width; x++)
-        {
-          int rx = x % paper_ppm.width;
 
-          for (y = 0; y < tmp.height; y++)
+      dx = runningvals.general_paint_edges ? paper_ppm.width - maxbrushwidth : 0;
+      dy = runningvals.general_paint_edges ? paper_ppm.height - maxbrushheight : 0;
+
+      for (y = 0; y < tmp.height; y++)
+        {
+          int lx;
+          int ry = (y + dy) % paper_ppm.height;
+
+          for (x = 0; x < tmp.width; x+=lx)
             {
-              int ry = y % paper_ppm.height;
+              int rx = (x + dx) % paper_ppm.width;
+
+              lx = MIN (tmp.width - x, paper_ppm.width - rx);
+
               memcpy (&tmp.col[y * tmp.width * 3 + x * 3],
-                      &paper_ppm.col[ry*paper_ppm.width*3+rx*3],
-                      3);
+                      &paper_ppm.col[ry * paper_ppm.width * 3 + rx * 3],
+                      3 * lx);
             }
         }
     }
@@ -790,9 +800,8 @@ repaint (ppm_t *p, ppm_t *a)
     {
       i = (int)(tmp.width * density / maxbrushwidth) *
           (int)(tmp.height * density / maxbrushheight);
-      step = i;
 #if 0
-    g_printerr("step=%d i=%d\n", step, i);
+    g_printerr("i=%d\n", i);
 #endif
     }
 
@@ -831,6 +840,9 @@ repaint (ppm_t *p, ppm_t *a)
 
   for (; i; i--)
     {
+      int n;
+      double thissum;
+
       if (i % progstep == 0)
         {
           if(runningvals.run)
@@ -886,7 +898,7 @@ repaint (ppm_t *p, ppm_t *a)
             continue;
         }
 
-      n = sn = on = 0;
+      sn = on = 0;
 
       switch (runningvals.orient_type)
         {
@@ -1022,7 +1034,7 @@ repaint (ppm_t *p, ppm_t *a)
       if (runningvals.color_noise > 0.0)
         {
           double v = runningvals.color_noise;
-#define BOUNDS(a) (((a) < 0) ? (a) : ((a) > 255) ? 255 : (a))
+#define BOUNDS(a) (((a) < 0) ? (0) : ((a) > 255) ? 255 : (a))
 #define MYASSIGN(a) \
     { \
         a = a + g_rand_double_range (random_generator, -v/2.0, v/2.0); \
